var checks = {

// This object defines a number of tests for the password.
// Any test results in a summand for the overall score with the following meaning:
// 0 = neutral
// negative score = leak in password. A leak should be measured against the length of the input string. One character means +1 point (see first function)
// -Infinity = password is useless
// positive score = there is a property, that is able to make the password very secure
//
// every function adds one number to the PROPS array. The key is a string identifying the test. The value should be 1 for PASSED, 0 for FAILED and 0.5 for NOTEXECUTED

checkLength : function(pw, props) {
  // check length of password
  // this is the basis of the algorithm, as it defines the weight for single characters:
  // - the first 4 characters obligatory
  // - the next 4 characters give 1/4 point each (meaning that 8 characters is neutral)
  // - any additional charachter gives 0.2 points. This is the minimum weight of one character and adding one should never lead to substraction of more than 0.2 points!
  var score = 0;
  if(pw.length < 4) {
      // 3 chars or less: useless
      score = -Infinity;
  } else {

      score = -4 // start with -4
      		  + Math.max(pw.length-4,0)*1; // and add 1 for every character (except the first 4)
  }
  if(pw.length < 8) {
    props["ccount"] = 0;
  } else {
    props["ccount"] = 1;
  }

  return score;
},

checkDigits : function(pw, props) {
  var score = 0;
  var regexres = pw.match(/\d/g);
  if(!$.isArray(regexres) || regexres.length==0) {
  	// the password has no digit. it takes 4-5 to repair this
    score = -3;
    props["digits"] = 0;
  } else {
    // ok. give some extra score for extra digits.
    props["digits"] = 1;
    score = (regexres.length-1)*0.625;
  }
  return score;
},

checkNonAlphanum : function(pw, props) {
  var score = 0;
  var regexres = pw.match(/\W|_/g);
  if(!$.isArray(regexres) || regexres.length==0) {
   	// the password has no special characters. it takes 4-5 to repair this
    score = -4;
    props["special"] = 0;
  } else {
    // ok. give some extra score for extra special chars.
    props["special"] = 1;
    score = (regexres.length-1)*0.75;
  }
  return score;
},

checkUpper : function(pw, props) {
  var score = 0;
  if(pw.match(/[A-Z]/)) {
    // OK, contains upper case letters
    score = 0;
  	props["uppercase"] = 1;
  } else {
    // bad
    score = -3;
    props["uppercase"] = 0;
  }
  return score;
},

checkLower : function(pw, props) {
  var score = 0;
  if(pw.match(/[a-z]/)) {
      // OK, contains lower case letters
    score = 0;
    props["lowercase"] = 1;
  } else {
    // bad
    score = -3;
    props["lowercase"] = 0;
  }
  return score;
},

checkRow : function(pw, props) {
  var score = 1;
  var rowcount = 0;
  for(i = 0; i < pw.length-2; i++) {
    if ( (Math.abs(pw.charCodeAt(i) - pw.charCodeAt(i+1)) < 3)
      && ( pw.charCodeAt(i) - pw.charCodeAt(i+1) == pw.charCodeAt(i+1) - pw.charCodeAt(i+2) )
    ) {
      rowcount++;
    }
  }
  if(rowcount) {
    score = -0.5*rowcount; // substract for every row (less than 1!)
  	props["norow"] = 0;
  } else {
    // no row. OK
  	score = 0;
  	props["norow"] = 1;
  }
  return score;
},

checkRepetition : function(pw, props) {
  var double = 0;
  var triple = 0;
  for(i = 0; i < pw.length; i++) {
    var regexp;
    // make the current charecter an regexp (escpace ?, +, $, ...)
    var regexpsyntaxcharsmatcher =/(\^|\$|\(|\)|\.|\[|\]|\+|\*|\?|\{|\}|\|\/)/;
    if(pw.charAt(i).search(regexpsyntaxcharsmatcher) != -1) {
    	var tmp = "\\";
    	regexp = eval("/"+tmp+pw.charAt(i)+"/g")
    } else {
    	regexp = eval("/"+pw.charAt(i)+"/g");
    }
    var regexpres = pw.match(regexp);
    if($.isArray(regexpres) && regexpres.length >= 2) {
      if(regexpres.length >= 3) {
        triple++;
      } else {
      	double++;
      }
    }
  }
  double /= 2; // every pair is counted twice
  triple /= 3; // every triple is counted 3x

  if(double >= 2 || triple >= 1) {
      // substract for every triple/pair
  	  props["norepetition"] = 0;
	  return -double*0.5-triple*0.75;
  } else {
    // no rep. OK.
    props["norepetition"] = 1;
	return 0;
  }

},

checkDict : function(pw, props, internet, languages) {
  // if INTERNET is set, this performs ajax request for XY.wiktionary.org where XY goes through all LANGUAGES
  var score = 0;
  if(!internet || (languages.length == 0) ) {
     // did not perform test
     props["nodict"] = 0.5;
     return 0;
  }
  if(pw.length == 0) {
  	 props["nodict"] = 1;
     return 0;
  }

  var hitcount = 0;
  var gotallresp = 1; // could all servers be reached
  $.each(languages, function(k,language) {
    var wikiresp = $.ajax({
      url: "http://"+language+".wiktionary.org/w/api.php?action=opensearch&search="+encodeURIComponent(pw)+"&namespace=0&suggest",
      async: false,
    });
    if(wikiresp.readyState == 4) {
      var hits = (eval(wikiresp.responseText))[1]; // all suggestions. get those which really fit:
      $.each(hits, function(key,value) {
      	if(value.toLowerCase().indexOf(pw.toLowerCase()) != -1) {
      		hitcount++;
      		return false; // end this for-loop, one hit is enough
      	}
      });
      if(hitcount) return false; // end for-loop, one hit is enough
    } else {
      // could not connect to a server
      gotallresp = 0;
    }
  });

  if (hitcount) {
    props["nodict"] = 0; // Hit! PW useless.
    return -Infinity;
  } else {
    if(gotallresp) {
      // no hit in all LANGUAGES
      props["nodict"] = 1;
      return 0;
    } else {
      // could not perform all tests => neutral (0), but 0.5 (not performed)
      props["nodict"] = 0.5;
      return 0;
    }
  }
}

}

// Do all tests, sum up the score and categorize the result in one of 5 categories.
function checkPW(pw, internet, languages) {
  var score = 0;
  var props = [];
  var debugstr = "";
  $.each(checks, function(funcname,func) {
    var testscore = func(pw, props, internet, languages);
  	score += testscore;
  	debugstr += funcname+' - '+testscore+'<br />';
  });
  debugstr += 'score = ' + score;
  $('#debug').html(debugstr);

  var category;
  if(score > 0) {
    category = 4;		// very strong
  } else if(score >= -3) {
    category = 3;		// strong
  } else if(score >= -6) {
    category = 2;		// medium
  } else if(score > -Infinity) {
    category = 1;		// weak
  } else {
    category = 0;		// useless
  }

  return [category, props];
}
